@@ -34,10 +34,6 @@
 #include "xf86_OSlib.h"
 #include "xf86OSpriv.h"
 
-#ifndef MAP_FAILED
-#define MAP_FAILED ((caddr_t)-1)
-#endif
-
 /***************************************************************************/
 /* Video Memory Mapping section                                            */
 /***************************************************************************/
@@ -48,44 +44,42 @@ static void sparc64UnmapVidMem(int, pointer, unsigned long);
 void
 xf86OSInitVidMem(VidMemInfoPtr pVidMem)
 {
-	pVidMem->linearSupported = TRUE;
-	pVidMem->mapMem = sparc64MapVidMem;
-	pVidMem->unmapMem = sparc64UnmapVidMem;
-	pVidMem->initialised = TRUE;
+    pVidMem->linearSupported = TRUE;
+    pVidMem->mapMem = sparc64MapVidMem;
+    pVidMem->unmapMem = sparc64UnmapVidMem;
+    pVidMem->initialised = TRUE;
 }
 
 static pointer
-sparc64MapVidMem(int ScreenNum, unsigned long Base, unsigned long Size, 
-		 int flags)
+sparc64MapVidMem(int ScreenNum, unsigned long Base, unsigned long Size,
+                 int flags)
 {
-	int fd = xf86Info.screenFd;
-	pointer base;
+    int fd = xf86Info.consoleFd;
+    pointer base;
 
 #ifdef DEBUG
-	xf86MsgVerb(X_INFO, 3, "mapVidMem %lx, %lx, fd = %d", 
-		    Base, Size, fd);
+    xf86MsgVerb(X_INFO, 3, "mapVidMem %lx, %lx, fd = %d", Base, Size, fd);
 #endif
 
-	base = mmap(0, Size,
-		    (flags & VIDMEM_READONLY) ?
-		     PROT_READ : (PROT_READ | PROT_WRITE),
-		    MAP_SHARED, fd, Base);
-	if (base == MAP_FAILED)
-		FatalError("%s: could not mmap screen [s=%x,a=%x] (%s)",
-			   "xf86MapVidMem", Size, Base, strerror(errno));
-	return base;
+    base = mmap(0, Size,
+                (flags & VIDMEM_READONLY) ?
+                PROT_READ : (PROT_READ | PROT_WRITE), MAP_SHARED, fd, Base);
+    if (base == MAP_FAILED)
+        FatalError("%s: could not mmap screen [s=%x,a=%x] (%s)",
+                   "xf86MapVidMem", Size, Base, strerror(errno));
+    return base;
 }
 
 static void
 sparc64UnmapVidMem(int ScreenNum, pointer Base, unsigned long Size)
 {
-	munmap(Base, Size);
+    munmap(Base, Size);
 }
 
 int
 xf86ReadBIOS(unsigned long Base, unsigned long Offset, unsigned char *Buf,
-	     int Len)
+             int Len)
 {
 
-	return (0);
+    return 0;
 }
